from typing import Dict, Union

from ..roiset import PatchParams
from ..session import session
from .shared import call_pipeline, PipelineParams, PipelineRecord, PipelineQueueRecord


class RoiSetPipelineParams(PipelineParams):
    roiset_index: Dict[str, int]
    patch_params: Dict[str, PatchParams] = {}


def call_roiset_pipeline(func, p: RoiSetPipelineParams) -> Union[PipelineRecord, PipelineQueueRecord]:
    def outer(x):
        trace, rois = func(x)
        session.add_roiset(
            roiset=rois,
            index_dict=p.roiset_index,
            patch_params=p.patch_params
        )
        return trace
    return call_pipeline(outer, p)